"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var redis_service_1 = require("../db/redis.service");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var cognitive_analysis_implementation_1 = require("./cognitive-analysis.implementation");
var logger = log4js.getLogger('Task');
var ROOT_KEY = "CognitiveEngineMigrationModel";
var ITEMS_PER_PAGE = 2;
var AVAILABLE = 'AVAILABLE';
var MIGRATING = 'MIGRATING';
var CANNOT_SAVE_DATA_TO_REDIS = 'CANNOT_SAVE_DATA_TO_REDIS';
var WITH_ERROR = 'WITH_ERROR';
var CULTURES = {
    es: 'es-es',
    'es-es': 'es',
    en: 'en-us',
    'en-us': 'en',
    fr: 'fr-fr',
    'fr-fr': 'fr',
    'es-mx': 'es',
};
var CognitiveEngineMigrateImplementation = (function () {
    function CognitiveEngineMigrateImplementation() {
    }
    CognitiveEngineMigrateImplementation.prototype.Execute = function (executionCode) {
        var _this = this;
        logger.info('[CognitiveEngineMigrateImplementation::Execute] executionCode:', executionCode);
        return promises_shared_1.DummyPromise().then(function (_) {
            return redis_service_1.Redis.GetCognitiveEngineMigrationData();
        })
            .then(function (results) {
            logger.info('[CognitiveEngineMigrateImplementation::Execute] FOUND DATA TO MIGRATE: ', results);
            if (typeof results != "object" || !results) {
                throw 'NOT_FOUND_DATA';
            }
            var promises = [];
            for (var vcc in results) {
                var data = JSON.parse(results[vcc]);
                switch (data.state) {
                    case 0:
                        promises.push(_this.StartMigration(vcc, data));
                        break;
                    case 1:
                        promises.push(_this.Migrate(vcc, data));
                        break;
                    default: break;
                }
            }
            return Promise.all(promises);
        })
            .then(function (results) {
            var hasSuccess = !results.includes(false);
            return hasSuccess;
        })
            .catch(function (err) {
            logger.error('[CognitiveEngineMigrateImplementation::Execute] Error ocurred on migrate', err);
            return false;
        });
    };
    CognitiveEngineMigrateImplementation.prototype.StartMigration = function (vcc, data) {
        var _this = this;
        return mssql_service_1.MSSql.GetNluIntentUtteranceToExport(vcc, 1).then(function (result) {
            var promises = [];
            if (result.nlus.length > 0) {
                promises.push(_this.SaveDataToRedis(vcc, result.nlus, 'Nlus', 'name'));
            }
            if (result.intents.length > 0) {
                promises.push(_this.SaveDataToRedis(vcc, result.intents, 'Intents', 'name'));
                if (result.utterances.length > 0) {
                    promises.push(_this.SaveDataToRedis(vcc, result.utterances, 'Utterances', 'id'));
                }
            }
            if (result.entities.length > 0) {
                promises.push(_this.SaveDataToRedis(vcc, result.entities, 'Entities', 'id'));
            }
            data.totalNlu = result.nlus.length;
            data.totalIntent = result.intents.length;
            data.totalUtterance = result.utterances.length;
            data.totalEntity = result.entities.length;
            return Promise.all(promises);
        }).then(function (results) {
            var errors = results.filter(function (r) { return r.includes("ERROR"); });
            if (errors.length > 0) {
                logger.error('[CognitiveEngineMigrateImplementation::StartMigration] AFTER SAVE DATA TO REDIS:', errors);
                return Promise.all([
                    false,
                    mssql_service_1.MSSql.SaveCognitiveEngineMigrateStatus(vcc, '*', 3, CANNOT_SAVE_DATA_TO_REDIS)
                ]);
            }
            return Promise.all([
                true,
                redis_service_1.Redis.UpdateCognitiveEngineMigrationData(vcc, Object.assign(data, { state: 1 })),
                mssql_service_1.MSSql.SaveCognitiveEngineMigrateStatus(vcc, '*', 2, MIGRATING)
            ]);
        }).then(function (status) {
            return status[0];
        }).catch(function (err) {
            logger.error('[CognitiveEngineMigrateImplementation::StartMigration] Error ocurred on start migration:', err);
            return mssql_service_1.MSSql.SaveCognitiveEngineMigrateStatus(vcc, '*', 3, WITH_ERROR);
        });
    };
    CognitiveEngineMigrateImplementation.prototype.SaveDataToRedis = function (vcc, rows, key, id) {
        return redis_service_1.Redis.SaveCognitiveEngineMigrationModel(ROOT_KEY + ":" + vcc + ":" + key, id, rows).then(function (saved) {
            return saved ? "SUCCESS_SAVE_" + key + "_TO_REDIS" : "ERROR_ON_SAVE_" + key + "_TO_REDIS";
        });
    };
    CognitiveEngineMigrateImplementation.prototype.Migrate = function (vcc, data) {
        var _this = this;
        return promises_shared_1.DummyPromise().then(function (_) {
            logger.info('[CognitiveEngineMigrateImplementation::Migrate] Getting Cognitive Service Instance For:', data.new);
            return cognitive_analysis_implementation_1.GetCognitiveService(null, data.new);
        })
            .then(function (service) {
            if (service) {
                logger.info('[CognitiveEngineMigrateImplementation::Migrate]  CognitiveEngine OK');
                return _this.MigrateNlus(vcc, data, 0, ITEMS_PER_PAGE, service);
            }
            logger.error('[CognitiveEngineMigrateImplementation::Migrate] ERROR_COGNITIVE_ENGINE_NOT_INSTANTIATED:', service);
            return 'ERROR_COGNITIVE_ENGINE_NOT_INSTANTIATED';
        }).then(function (result) {
            if (typeof result === 'string' && result.includes("ERROR")) {
                logger.error('[CognitiveEngineMigrateImplementation::Migrate] Found errors on migrate:', result);
                return _this.ProcessErrors(vcc, result);
            }
            return mssql_service_1.MSSql.SaveCognitiveEngineMigrateStatus(vcc, '*', 1, AVAILABLE);
        });
    };
    CognitiveEngineMigrateImplementation.prototype.MigrateNlus = function (vcc, data, start, end, service) {
        var _this = this;
        if (start === void 0) { start = 0; }
        if (end === void 0) { end = 1; }
        return promises_shared_1.DummyPromise()
            .then(function (_) {
            return redis_service_1.Redis.LRange(ROOT_KEY + ":" + vcc + ":Nlus:Keys", start, end - 1);
        })
            .then(function (nluKeys) {
            var promises = [];
            if (!nluKeys || nluKeys.length === 0) {
                promises.push(Promise.resolve(false));
                promises.push(redis_service_1.Redis.UpdateCognitiveEngineMigrationData(vcc, Object.assign(data, { state: 2 })));
            }
            else {
                promises.push(Promise.resolve(true));
                promises.push(redis_service_1.Redis.GetCognitiveEngineMigrationModel(ROOT_KEY + ":" + vcc + ":Nlus", nluKeys));
            }
            return Promise.all(promises);
        })
            .then(function (results) {
            if (!results[0]) {
                return false;
            }
            var nlus = results[1];
            if (nlus) {
                var nluList = [];
                for (var _i = 0, nlus_1 = nlus; _i < nlus_1.length; _i++) {
                    var nluStr = nlus_1[_i];
                    var nlu = JSON.parse(nluStr);
                    nluList.push(Object.assign(nlu, {
                        culture: CULTURES[nlu.culture]
                    }));
                }
                return _this.ProcessNlus(vcc, data, nluList, service);
            }
            return false;
        })
            .then(function (_) {
            if (end < data.totalNlu) {
                return _this.MigrateNlus(vcc, data, start + ITEMS_PER_PAGE, end + ITEMS_PER_PAGE, service);
            }
            return false;
        })
            .catch(function (err) {
            return false;
        });
    };
    CognitiveEngineMigrateImplementation.prototype.ProcessNlus = function (vcc, data, nlus, service) {
        var _this = this;
        return promises_shared_1.DummyPromise().then(function (_) {
            var promises = [];
            for (var _i = 0, nlus_2 = nlus; _i < nlus_2.length; _i++) {
                var nlu = nlus_2[_i];
                switch (nlu.status) {
                    case 0:
                        promises.push(_this.AddApp(vcc, data, nlu, service));
                        break;
                    case 1:
                        promises.push(_this.DeleteApp(vcc, data, nlu));
                        break;
                    default: break;
                }
            }
            return Promise.all(promises);
        }).then(function (results) {
            return true;
        });
    };
    CognitiveEngineMigrateImplementation.prototype.AddApp = function (vcc, data, nlu, service) {
        return promises_shared_1.DummyPromise().then(function (_) {
            return service.AddApp(data.subscriptionRegion, data.subscriptionKey, Object.assign(nlu, {
                vcc: vcc
            }));
        }).then(function (newAppId) {
            logger.info('[CognitiveEngineMigrateImplementation::AddApp] AppIds ', newAppId);
            if (newAppId) {
                var promises = [];
                promises.push(mssql_service_1.MSSql.UpdateNluAppId(vcc, nlu.appId, newAppId));
                promises.push(redis_service_1.Redis.SaveCognitiveEngineMigrationNewId(ROOT_KEY + ":" + vcc + ":AppIds", nlu.appId, newAppId));
                return Promise.all(promises);
            }
            return [false];
        }).then(function (results) {
            logger.info('[CognitiveEngineMigrateImplementation::AddApp] UPDATE APP ID:', results);
            if (!results.includes(false)) {
                var promises = [];
                promises.push(redis_service_1.Redis.UpdateCognitiveEngineMigrationModel(ROOT_KEY + ":" + vcc + ":Nlus", nlu.name, Object.assign(nlu, {
                    status: 1
                })));
                Promise.all(promises);
            }
            return [false];
        }).then(function (results) {
            return !results.includes(false);
        });
    };
    CognitiveEngineMigrateImplementation.prototype.DeleteApp = function (vcc, data, nlu) {
        return promises_shared_1.DummyPromise().then(function (_) {
            return cognitive_analysis_implementation_1.GetCognitiveService(null, data.old);
        }).then(function (service) {
            if (service) {
                return service.DeleteApp(data.subscriptionRegion, data.subscriptionKey, Object.assign(nlu, {
                    vcc: vcc,
                }));
            }
            logger.error('[CognitiveEngineMigrateImplementation::DeleteApp] NOT_FOUND_OLD_COGNITIVE_ENGINE', service);
            return false;
        }).then(function (deleted) {
            if (deleted) {
                return redis_service_1.Redis.DeleteCognitiveEngineMigrationModel(ROOT_KEY + ":" + vcc + ":Nlus", nlu.name);
            }
            return false;
        });
    };
    CognitiveEngineMigrateImplementation.prototype.MigrateIntents = function (vcc, data, start, end, service) {
        var _this = this;
        return promises_shared_1.DummyPromise().then(function (_) {
            return redis_service_1.Redis.LRange(ROOT_KEY + ":" + vcc + ":Intents:Keys", start, end - 1);
        }).then(function (intentKeys) {
            if (!intentKeys) {
                return false;
            }
            return redis_service_1.Redis.GetCognitiveEngineMigrationModel(ROOT_KEY + ":" + vcc + ":Intents", intentKeys);
        }).then(function (intents) {
            if (intents) {
                var intentList = [];
                for (var _i = 0, intents_1 = intents; _i < intents_1.length; _i++) {
                    var i = intents_1[_i];
                    var intent = JSON.parse(i);
                    intentList.push(intent);
                }
                return _this.ProcessIntents(vcc, data, intentList, service);
            }
            return false;
        }).then(function (results) {
            if (end < data.totalIntent) {
                return _this.MigrateNlus(vcc, data, start + ITEMS_PER_PAGE, end + ITEMS_PER_PAGE, service);
            }
            return false;
        });
    };
    CognitiveEngineMigrateImplementation.prototype.ProcessIntents = function (vcc, data, intents, service) {
        var _this = this;
        return promises_shared_1.DummyPromise().then(function (_) {
            var promises = [];
            for (var _i = 0, intents_2 = intents; _i < intents_2.length; _i++) {
                var intent = intents_2[_i];
                switch (intent.status) {
                    case 0:
                        promises.push(_this.AddIntent(vcc, data, intent, service));
                        break;
                    case 1:
                        promises.push(_this.DeleteIntent(vcc, data, intent));
                        break;
                    default: break;
                }
            }
            return Promise.all(promises);
        }).then(function (results) {
            return true;
        });
    };
    CognitiveEngineMigrateImplementation.prototype.AddIntent = function (vcc, data, intent, service) {
        var _this = this;
        var AppId = null;
        return promises_shared_1.DummyPromise().then(function (_) {
            return redis_service_1.Redis.GetCognitiveEngineMigrationNewId(ROOT_KEY + ":" + vcc + ":AppIds", intent.appId);
        }).then(function (appId) {
            if (appId) {
                AppId = appId;
                return service.AddIntent(data.subscriptionRegion, data.subscriptionKey, appId, intent.name);
            }
            logger.error('[CognitiveEngineMigrateImplementation::AddIntent] Error on get new intent id from redis ', intent);
            return null;
        }).then(function (newIntentId) {
            if (newIntentId) {
                return mssql_service_1.MSSql.UpdateIntentId(vcc, AppId, intent.intentId, newIntentId);
            }
            logger.error("[CognitiveEngineMigrateImplementation::AddIntent] Error on add intent into " + data.new + ":", newIntentId);
            return false;
        })
            .then(function (result) {
            logger.info('[CognitiveEngineMigrateImplementation::AddIntent] UPDATE APP ID:', result);
            if (result) {
                var promises = [];
                promises.push(redis_service_1.Redis.UpdateCognitiveEngineMigrationModel(ROOT_KEY + ":" + vcc + ":Intents", intent.name, Object.assign(intent, {
                    status: 1
                })));
                if (data.totalUtterance > 0) {
                    promises.push(_this.MigrateUtterances(vcc, data, intent.intentId, 0, ITEMS_PER_PAGE, service));
                }
                Promise.all(promises);
            }
            return [false];
        })
            .then(function (results) {
            return true;
        });
    };
    CognitiveEngineMigrateImplementation.prototype.DeleteIntent = function (vcc, data, intent) {
        return promises_shared_1.DummyPromise().then(function (_) {
            return cognitive_analysis_implementation_1.GetCognitiveService(null, data.old);
        }).then(function (service) {
            if (service) {
                return service.DeleteAppIntent(data.subscriptionRegion, data.subscriptionKey, intent.appId, intent.intentId);
            }
            logger.error('[CognitiveEngineMigrateImplementation::DeleteIntent] NOT_FOUND_OLD_COGNITIVE_ENGINE', service);
            return false;
        }).then(function (deleted) {
            if (deleted) {
                return redis_service_1.Redis.DeleteCognitiveEngineMigrationModel(ROOT_KEY + ":" + vcc + ":Intents", intent.name);
            }
            return false;
        });
    };
    CognitiveEngineMigrateImplementation.prototype.MigrateUtterances = function (vcc, data, intentId, start, end, service) {
        var _this = this;
        return promises_shared_1.DummyPromise().then(function (_) {
            return redis_service_1.Redis.LRange(ROOT_KEY + ":" + vcc + ":Utterances:Keys", start, end - 1);
        }).then(function (Keys) {
            if (!Keys) {
                return false;
            }
            return redis_service_1.Redis.GetCognitiveEngineMigrationModel(ROOT_KEY + ":" + vcc + ":Utterances", Keys);
        }).then(function (utterances) {
            if (utterances) {
                var utteranceList = [];
                for (var _i = 0, utterances_1 = utterances; _i < utterances_1.length; _i++) {
                    var u = utterances_1[_i];
                    var utterance = JSON.parse(u);
                    utteranceList.push(utterance);
                }
            }
            return false;
        }).then(function (results) {
            if (end < data.totalUtterance) {
                return _this.MigrateUtterances(vcc, data, intentId, start + ITEMS_PER_PAGE, end + ITEMS_PER_PAGE, service);
            }
            return false;
        });
    };
    CognitiveEngineMigrateImplementation.prototype.ProcessUtterances = function (vcc, data, intentId, utterances, service) {
        var _this = this;
        return promises_shared_1.DummyPromise().then(function (_) {
            var promises = [];
            for (var _i = 0, utterances_2 = utterances; _i < utterances_2.length; _i++) {
                var utterance = utterances_2[_i];
                switch (utterance.status) {
                    case 0:
                        promises.push(_this.AddUtterance(vcc, data, utterance, service));
                        break;
                    case 1:
                        break;
                    default: break;
                }
            }
            return Promise.all(promises);
        }).then(function (results) {
            return true;
        });
    };
    CognitiveEngineMigrateImplementation.prototype.AddUtterance = function (vcc, data, utterance, service) {
        var AppId = null;
        return promises_shared_1.DummyPromise().then(function (_) {
            return redis_service_1.Redis.GetCognitiveEngineMigrationNewId(ROOT_KEY + ":" + vcc + ":AppIds", utterance.appId);
        })
            .then(function (appId) {
            if (appId) {
                AppId = appId;
                return service.AddUtterance(data.subscriptionRegion, data.subscriptionKey, appId, utterance);
            }
            logger.error('[CognitiveEngineMigrateImplementation::AddUtterance] Error on get new app id from redis ', utterance);
            return null;
        }).then(function (newUtteranceId) {
            if (newUtteranceId) {
                return mssql_service_1.MSSql.UpdateUtteranceId(vcc, AppId, utterance.intent, utterance.id, newUtteranceId);
            }
            logger.error("[CognitiveEngineMigrateImplementation::AddUtterance] Error on add utterance into " + data.new + ":", newUtteranceId);
            return false;
        })
            .then(function (updated) {
            logger.info('[CognitiveEngineMigrateImplementation::AddIntent] UPDATE APP ID:', updated);
            if (updated) {
                return redis_service_1.Redis.UpdateCognitiveEngineMigrationModel(ROOT_KEY + ":" + vcc + ":Utterances", utterance.id, Object.assign(utterance, {
                    status: 1
                }));
            }
            return false;
        })
            .then(function (results) {
            return true;
        });
    };
    CognitiveEngineMigrateImplementation.prototype.DeleteUtterance = function (vcc, data, utterance) {
        return promises_shared_1.DummyPromise().then(function (_) {
            return cognitive_analysis_implementation_1.GetCognitiveService(null, data.old);
        }).then(function (service) {
            if (service) {
                return service.DeleteAppUtterance(data.subscriptionRegion, data.subscriptionKey, utterance.appId, utterance.intent, utterance.id);
            }
            logger.error('[CognitiveEngineMigrateImplementation::DeleteIntent] NOT_FOUND_OLD_COGNITIVE_ENGINE', service);
            return false;
        }).then(function (deleted) {
            if (deleted) {
                return redis_service_1.Redis.DeleteCognitiveEngineMigrationModel(ROOT_KEY + ":" + vcc + ":Utterances", utterance.id);
            }
            return false;
        });
    };
    CognitiveEngineMigrateImplementation.prototype.MigrateEntities = function (vcc, data, start, end, service) {
        var _this = this;
        return promises_shared_1.DummyPromise().then(function (_) {
            return redis_service_1.Redis.LRange(ROOT_KEY + ":" + vcc + ":Entities:Keys", start, end - 1);
        }).then(function (keys) {
            if (!keys) {
                return false;
            }
            return redis_service_1.Redis.GetCognitiveEngineMigrationModel(ROOT_KEY + ":" + vcc + ":Entities", keys);
        }).then(function (entities) {
            if (entities) {
                var entityList = [];
                for (var _i = 0, entities_1 = entities; _i < entities_1.length; _i++) {
                    var e = entities_1[_i];
                    var entity = JSON.parse(e);
                    entityList.push(entity);
                }
                return _this.ProcessEntities(vcc, data, entityList, service);
            }
            return false;
        }).then(function (results) {
            if (end < data.totalEntity) {
                return _this.MigrateNlus(vcc, data, start + ITEMS_PER_PAGE, end + ITEMS_PER_PAGE, service);
            }
            return false;
        });
    };
    CognitiveEngineMigrateImplementation.prototype.ProcessEntities = function (vcc, data, entities, service) {
        var _this = this;
        return promises_shared_1.DummyPromise().then(function (_) {
            var promises = [];
            for (var _i = 0, entities_2 = entities; _i < entities_2.length; _i++) {
                var entity = entities_2[_i];
                switch (entity.status) {
                    case 0:
                        promises.push(_this.AddEntity(vcc, data, entity, service));
                        break;
                    case 1:
                        promises.push(_this.DeleteEntity(vcc, data, entity));
                        break;
                    default: break;
                }
            }
            return Promise.all(promises);
        }).then(function (results) {
            return true;
        });
    };
    CognitiveEngineMigrateImplementation.prototype.AddEntity = function (vcc, data, entity, service) {
        var AppId = null;
        return promises_shared_1.DummyPromise().then(function (_) {
            return redis_service_1.Redis.GetCognitiveEngineMigrationNewId(ROOT_KEY + ":" + vcc + ":AppIds", entity.appId);
        }).then(function (appId) {
            if (appId) {
                AppId = appId;
                return service.AddSimpleEntity(data.subscriptionRegion, data.subscriptionKey, appId, entity);
            }
            logger.error('[CognitiveEngineMigrateImplementation::AddEntity] Error on get new app id from redis ', entity);
            return null;
        }).then(function (newId) {
            if (newId) {
                return mssql_service_1.MSSql.UpdateEntityId(vcc, AppId, entity.id, newId);
            }
            logger.error("[CognitiveEngineMigrateImplementation::AddEntity] Error on add intent into " + data.new + ":", newId);
            return false;
        })
            .then(function (result) {
            logger.info('[CognitiveEngineMigrateImplementation::AddEntity] UPDATE APP ID:', result);
            if (result) {
                return redis_service_1.Redis.UpdateCognitiveEngineMigrationModel(ROOT_KEY + ":" + vcc + ":Entities", entity.id, Object.assign(entity, {
                    status: 1
                }));
            }
            return false;
        })
            .then(function (results) {
            return true;
        });
    };
    CognitiveEngineMigrateImplementation.prototype.DeleteEntity = function (vcc, data, entity) {
        return promises_shared_1.DummyPromise().then(function (_) {
            return cognitive_analysis_implementation_1.GetCognitiveService(null, data.old);
        }).then(function (service) {
            if (service) {
                return service.DeleteAppEntity(data.subscriptionRegion, data.subscriptionKey, entity.appId, entity.id);
            }
            logger.error('[CognitiveEngineMigrateImplementation::DeleteEntity] NOT_FOUND_OLD_COGNITIVE_ENGINE', service);
            return false;
        }).then(function (deleted) {
            if (deleted) {
                return redis_service_1.Redis.DeleteCognitiveEngineMigrationModel(ROOT_KEY + ":" + vcc + ":Entities", entity.id);
            }
            return false;
        });
    };
    CognitiveEngineMigrateImplementation.prototype.ProcessErrors = function (vcc, error) {
        var promises = [];
        switch (error) {
            case 'ERROR_COGNITIVE_ENGINE_NOT_INSTANTIATED':
                promises.push(mssql_service_1.MSSql.SaveCognitiveEngineMigrateStatus(vcc, '*', 3, 'ERROR_COGNITIVE_ENGINE_NOT_INSTANTIATED'));
                break;
            default: break;
        }
        return Promise.all(promises).then(function (_) {
            return false;
        });
    };
    return CognitiveEngineMigrateImplementation;
}());
exports.CognitiveEngineMigrateImplementation = CognitiveEngineMigrateImplementation;
